/*
 * Xenoborg - Xbox Emulator
 * Copyright (C) 2006-2010  blueshogun96
 *
 * Name: x86Op2.cpp
 * Desc: Handles 2 byte x86 opcodes.
 *
 * Changelog: 
 *	10/14/2010 - Initial import from an old source build.
 *
 *  10/16/2010 - Added WBINVD to get it out of the way early (not implemented yet
 *				 though). Maybe we can just ignore it...
 */


#include "..\\XenoKrnl.h"
#include "..\\EmuMMU.h"
#include "x86OpTables.h"

// Globals
extern x86Instruction Instruction;
extern LPEXCEPTION_POINTERS pE;

__inline int x86op_WBINVD( void )		// Opcode 0x0F 0x09
{
	// TODO: Anything?
	DbgPrintf( ">>> WBINVD instruction not yet implemented! <<<\n" );

	return 1;
}

__inline int x86op_Unimplemented_2byte( void )
{
	return 0;
}

const x86op x86_opcode_table_2byte[] = 
{
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//0 (0x0)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//1 (0x1)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//2 (0x2)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//3 (0x3)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//4 (0x4)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//5 (0x5)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//6 (0x6)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//7 (0x7)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//8 (0x8)
	{ x86op_WBINVD, x86op_WBINVD },								//9 (0x9)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//10 (0xa)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//11 (0xb)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//12 (0xc)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//13 (0xd)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//14 (0xe)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//15 (0xf)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//16 (0x10)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//17 (0x11)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//18 (0x12)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//19 (0x13)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//20 (0x14)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//21 (0x15)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//22 (0x16)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//23 (0x17)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//24 (0x18)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//25 (0x19)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//26 (0x1a)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//27 (0x1b)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//28 (0x1c)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//29 (0x1d)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//30 (0x1e)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//31 (0x1f)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//32 (0x20)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//33 (0x21)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//34 (0x22)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//35 (0x23)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//36 (0x24)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//37 (0x25)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//38 (0x26)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//39 (0x27)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//40 (0x28)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//41 (0x29)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//42 (0x2a)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//43 (0x2b)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//44 (0x2c)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//45 (0x2d)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//46 (0x2e)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//47 (0x2f)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//48 (0x30)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//49 (0x31)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//50 (0x32)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//51 (0x33)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//52 (0x34)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//53 (0x35)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//54 (0x36)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//55 (0x37)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//56 (0x38)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//57 (0x39)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//58 (0x3a)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//59 (0x3b)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//60 (0x3c)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//61 (0x3d)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//62 (0x3e)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//63 (0x3f)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//64 (0x40)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//65 (0x41)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//66 (0x42)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//67 (0x43)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//68 (0x44)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//69 (0x45)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//70 (0x46)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//71 (0x47)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//72 (0x48)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//73 (0x49)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//74 (0x4a)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//75 (0x4b)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//76 (0x4c)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//77 (0x4d)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//78 (0x4e)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//79 (0x4f)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//80 (0x50)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//81 (0x51)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//82 (0x52)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//83 (0x53)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//84 (0x54)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//85 (0x55)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//86 (0x56)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//87 (0x57)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//88 (0x58)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//89 (0x59)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//90 (0x5a)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//91 (0x5b)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//92 (0x5c)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//93 (0x5d)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//94 (0x5e)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//95 (0x5f)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//96 (0x60)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//97 (0x61)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//98 (0x62)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//99 (0x63)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//100 (0x64)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//101 (0x65)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//102 (0x66)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//103 (0x67)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//104 (0x68)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//105 (0x69)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//106 (0x6a)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//107 (0x6b)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//108 (0x6c)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//109 (0x6d)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//110 (0x6e)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//111 (0x6f)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//112 (0x70)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//113 (0x71)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//114 (0x72)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//115 (0x73)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//116 (0x74)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//117 (0x75)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//118 (0x76)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//119 (0x77)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//120 (0x78)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//121 (0x79)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//122 (0x7a)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//123 (0x7b)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//124 (0x7c)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//125 (0x7d)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//126 (0x7e)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//127 (0x7f)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//128 (0x80)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//129 (0x81)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//130 (0x82)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//131 (0x83)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//132 (0x84)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//133 (0x85)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//134 (0x86)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//135 (0x87)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//136 (0x88)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//137 (0x89)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//138 (0x8a)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//139 (0x8b)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//140 (0x8c)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//141 (0x8d)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//142 (0x8e)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//143 (0x8f)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//144 (0x90)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//145 (0x91)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//146 (0x92)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//147 (0x93)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//148 (0x94)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//149 (0x95)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//150 (0x96)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//151 (0x97)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//152 (0x98)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//153 (0x99)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//154 (0x9a)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//155 (0x9b)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//156 (0x9c)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//157 (0x9d)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//158 (0x9e)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//159 (0x9f)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//160 (0xa0)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//161 (0xa1)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//162 (0xa2)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//163 (0xa3)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//164 (0xa4)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//165 (0xa5)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//166 (0xa6)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//167 (0xa7)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//168 (0xa8)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//169 (0xa9)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//170 (0xaa)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//171 (0xab)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//172 (0xac)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//173 (0xad)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//174 (0xae)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//175 (0xaf)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//176 (0xb0)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//177 (0xb1)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//178 (0xb2)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//179 (0xb3)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//180 (0xb4)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//181 (0xb5)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//182 (0xb6)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//183 (0xb7)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//184 (0xb8)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//185 (0xb9)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//186 (0xba)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//187 (0xbb)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//188 (0xbc)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//189 (0xbd)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//190 (0xbe)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//191 (0xbf)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//192 (0xc0)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//193 (0xc1)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//194 (0xc2)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//195 (0xc3)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//196 (0xc4)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//197 (0xc5)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//198 (0xc6)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//199 (0xc7)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//200 (0xc8)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//201 (0xc9)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//202 (0xca)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//203 (0xcb)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//204 (0xcc)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//205 (0xcd)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//206 (0xce)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//207 (0xcf)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//208 (0xd0)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//209 (0xd1)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//210 (0xd2)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//211 (0xd3)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//212 (0xd4)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//213 (0xd5)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//214 (0xd6)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//215 (0xd7)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//216 (0xd8)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//217 (0xd9)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//218 (0xda)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//219 (0xdb)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//220 (0xdc)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//221 (0xdd)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//222 (0xde)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//223 (0xdf)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//224 (0xe0)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//225 (0xe1)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//226 (0xe2)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//227 (0xe3)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//228 (0xe4)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//229 (0xe5)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//230 (0xe6)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//231 (0xe7)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//232 (0xe8)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//233 (0xe9)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//234 (0xea)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//235 (0xeb)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//236 (0xec)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//237 (0xed)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//238 (0xee)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//239 (0xef)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//240 (0xf0)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//241 (0xf1)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//242 (0xf2)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//243 (0xf3)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//244 (0xf4)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//245 (0xf5)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//246 (0xf6)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//247 (0xf7)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//248 (0xf8)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//249 (0xf9)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//250 (0xfa)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//251 (0xfb)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//252 (0xfc)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//253 (0xfd)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//254 (0xfe)
	{ x86op_Unimplemented_2byte, x86op_Unimplemented_2byte },	//255 (0xff)
};
